plotNoise(TString fname, TString refHistoFile = "/home/mweber/richdigi/sep08_realnoise_part0.root",Float_t nrreffiles = 1.){
  
  gROOT->Reset();
  
  TChain ch("T");

  TFile* f0 = TFile::Open(fname);
  
  HTree *T = (HTree*)gDirectory->Get("T");
  if(!T) {cout << "No tree in file!" << endl; continue;}
  
  f0->cd("dirRich");
  
  //Rich Cal
  HRichCal* ptlHRichCal = new HRichCal();
  HLinearCategory* catlHRichCal = (HLinearCategory*) gDirectory->Get("HRichCalSim");


  if(catlHRichCal){
    TBranch* brlHRichCal = T->GetBranch("HRichCalSim.");
    if(brlHRichCal) brlHRichCal->SetAddress(&catlHRichCal);
		  }

  else
    {
      cout << "No RICHCal category" << endl;
    }
  
  if(catlHRichCal)TIterator* iterRichCal = catlHRichCal  ->MakeIterator("native");

  Int_t ientries = T->GetEntries();  // how many events
  cout << ientries << " Events" << endl;
  Int_t nbytes=0;

  TH1F *h0   = new TH1F("h0","cleaned single pads",1000,0,20);
  TH1F *hcal[6];
  TString hname[6]={"hcal1","hcal2","hcal3","hcal4","hcal5","hcal6"};

  for(Int_t j = 0; j< 6; ++j){
    hcal[j] = new TH1F(hname[j],"pads in sector",20,0,20);
  }

  Int_t nrcal[6];


  for (Int_t i=0;i<ientries;i++){ 

    nbytes = T->GetEntry(i);
    
    HRichCal*   pObjRichCal = NULL;
     
    if((i%500)==0){
      cout<<"EVT="<<i<<endl;
    }
    
    
    iterRichCal->Reset();
    for(Int_t j = 0; j< 6; ++j){
      nrcal[j] = 0;
    }
 

    while( (pObjRichCal=(HRichCal*)iterRichCal->Next())!=0){

      if(pObjRichCal->getIsCleanedSingle()==0) continue;
      
      h0->Fill(pObjRichCal->getCharge());
      nrcal[pObjRichCal->getSector()]++;
    }
    for(Int_t j = 0; j< 6; ++j){
      hcal[j]->Fill(nrcal[j]);
    }
  }

  //normalize to nr of events and sectors
  Float_t norm = (1./(6.*(Float_t)i));

  TCanvas *c = new TCanvas;
  TCanvas *d = new TCanvas;
  d->Divide(2,3);

  c->cd();
  h0->Scale(norm);
  h0->SetLineWidth(2);
  h0->GetXaxis()->SetTitle("pad amplitude ( ADC channel )");
  h0->GetYaxis()->SetTitle("counts/(events*sectors)");
  h0->DrawCopy();


  for(Int_t j = 0; j< 6; ++j){
    d->cd(j+1);   
    hcal[j]->SetLineWidth(2);
    hcal[j]->Scale(norm*6.);
    hcal[j]->DrawCopy();
  }

  f0->Close();

  TFile* f1 = TFile::Open(refHistoFile);
  TH1F * h1 = (TH1F*) f1->Get("hCharge");
  TH1F * href = (TH1F*) f1->Get("hCal");
  h1->SetLineColor(2);
  h1->SetLineWidth(2);
  h1->Scale(1/nrreffiles);
  href->SetLineColor(2);
  href->SetLineWidth(2);
  href->Scale(1/nrreffiles);
  c->cd();
  h1->DrawCopy("same");

  for(Int_t j = 0; j< 6; ++j){
    d->cd(j+1);   
    href->DrawCopy("same");
  }

  f1->Close();

}
