/**
 * @file   dst-embedding.cc
 * @author Simon Lang
 * @date   Mon Jun 11 22:06:01 2007
 *
 * @brief  This file can be compiled or executed within CINT to produce
 *         a DST file of simulated tracks embedding in real data.
 *         The program parameter can be provides via a text configuration file
 *         and/or on command line. If this function is called form CINT,
 *         then just provide the command line as a string argument.
 *         For details see example.cfg and the code of module "dst".
 *
 * $Id$
 *
 */
   
   
#ifndef __CINT__

#include "TROOT.h"
#include "TString.h"
#include "hades.h"
#include "hdstembedding.h"
#include "hdstutilities.h"
#include "hrichtaskset.h"


int main(int argc, char **argv)
{
   TROOT dst_production( "DstProductionApp", "Produce a DST File" );
   HDstEmbedding embedding;

   embedding.configure( argc, argv );

#else

void DstEmbedding(TString command_line)
{
   int                argc = 0;
   char**             argv = NULL;
   HDstEmbedding embedding;

   HDstUtilities::str2CmdLineArgs( command_line.Data(), &argc, &argv );
   embedding.configure( argc, argv );
   HDstUtilities::freeCmdLineArgs( argc, argv );

#endif

   embedding.printConfiguration();
   embedding.setupSpectrometer();
   embedding.setupInput();
   embedding.setupParameterSources();

   HRichTaskSet*    rich_task_set     = new HRichTaskSet();
   HTaskSet* tasks = gHades->getTaskSet( "real" );
   tasks->add( rich_task_set->make("real","") );
   tasks->isTimed( kTRUE );

   embedding.initialize();
   embedding.setupOutput();
   embedding.runEventLoop();
   embedding.finalize();
}
