using namespace std;
#ifndef __CINT__

#include "TROOT.h"
#include "TString.h"
#include "TSystem.h"

#include "hdstutilities.h"
#include "hdstsimulation.h"

#include "hades.h"
#include "htaskset.h"
#include "hruntimedb.h"
#include "hdatasource.h"

#include "hstarttaskset.h"
#include "htriggertaskset.h"

#include "hrichIPUtaskset.h"
#include "hrichtaskset.h"

#include "htofinotaskset.h"
#include "htoftaskset.h"

#include "hshowertaskset.h"

#include "hmdcbitflipcor.h"
#include "hmdcsetup.h"
#include "hmdctaskset.h"
#include "hmdclookuptb.h"
#include "hmdc34clfinder.h"
#include "hmdctrackdset.h"
#include "hmdcdigitpar.h"
#include "hmdcdigitizer.h"
#include "hmdcvertexwriter.h"
#include "hmdcvertexfind.h"

#include "hrktrackBF.h"
#include "hsplinetaskset.h"
#include "hkicktaskset.h"
#include "hkicktrackbaseF.h"

#include "hpidtofrec.h"
#include "hpidalgstandcuts.h"
#include "hpidparticlefiller.h"
#include "hpidpdfmaker.h"
#include "hpidreconstructor.h"
#include "hpidskiptrigger.h"
#include "hpidtrackfiller.h"
#include "hpidevtinfofiller.h"
#include "hpidtrackcleaner.h"

#include "hpairfiller.h"
#include "hpairfilter.h"

#include "hqamaker.h"

int main(int argc, char **argv)
{
   TROOT dst_production( "DstProductionApp", "Produce a DST File" );
   HDstSimulation simulation;
   
   simulation.configure( argc, argv );
#else


void DstSimulation(TString command_line)
{
   int                 argc = 0;
   char**              argv = NULL;
   HDstSimulation simulation;

   HDstUtilities::str2CmdLineArgs( command_line.Data(), &argc, &argv );
   simulation.configure( argc, argv );
   HDstUtilities::freeCmdLineArgs( argc, argv );
   
   
#endif

   simulation.printConfiguration();
   simulation.setupSpectrometer();
   simulation.setupInput();
   simulation.setupParameterSources();
   //simulation.setupParameterOutput();


   // TASK LIST

   // read-only access to the configuration
   const HDstProduction::DstConfiguration_t& dstCfg = simulation.getConfiguration();

   HRuntimeDb* rtdb=gHades->getRuntimeDb();


   HTriggerTaskSet* trigger_task_set  = new HTriggerTaskSet();
   HTofTaskSet*     tof_task_set      = new HTofTaskSet();
   HTofinoTaskSet*  tofino_task_set   = new HTofinoTaskSet();
   HRichTaskSet*    rich_task_set     = new HRichTaskSet();
   HRichIPUTaskSet* rich_ipu_task_set = new HRichIPUTaskSet();
   HMdcTaskSet*     mdc_task_set      = new HMdcTaskSet();
   HShowerTaskSet*  shower_task_set   = new HShowerTaskSet();
   HSplineTaskSet*  spline_task_set   = new HSplineTaskSet( "", "" );


   // use new MDC cluster finder with level adaption
   HMdcLookUpTb::setUseFloatLevel();        // inner MDCs (Apr06)
   HMdc34ClFinder::setUseFloatLevel();      // outer MDCs (Apr06)
   //HMdcTrackDSet::setMixCuts(1,2);          // Very important for mixed mode with low cluster settings (jan04) !


   // this includes the following tasks:
   // Kick Tracking, Kick Track Clustering,
   // Spline Tracking, Spline Clustering,
   // Runge-Kutta Tracking, MetaMatch Procedure
   HTask* tracking_tasks = spline_task_set->make("", "spline,reslowb,&old,tofclust,simulation,runge" );

    //-------------TOF reconstruction for PP data--------------------------
    // new version of the TofReconstructor, since v7_07
    HPidTofRec *pidtofrec = new HPidTofRec("pidtofreconstructor","pidtofreconstructor","nopidparam");
    pidtofrec->setQualityRichParams(200.,5.,5.,4.);  // RingPatMat, RingPadNr, Centroid, RingverageCharge


    //------------------------Particle identification----------------------
    TString   pid_ntuple;
    TString   pair_filt_ntuple;

    // derive output and NTuple file name from input file name
    pid_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pid_ntuple.ReplaceAll( "_dst.root", 9, "_lep_nt.root", 12 );
    pid_ntuple.Prepend( "/" );
    pid_ntuple.Prepend( dstCfg.outputDir );

    pair_filt_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pair_filt_ntuple.ReplaceAll( "_dst.root", 9, "_pair_filt_nt.root", 18 );
    pair_filt_ntuple.Prepend( "/" );
    pair_filt_ntuple.Prepend( dstCfg.outputDir );


    TString opt_pid_rec = "pdf,CL,merge,ALG_RUNGEKUTTA";
    HPidReconstructor *pid_rec = new HPidReconstructor(opt_pid_rec.Data());

    Short_t nParticles[2] = {2,3};
    pid_rec->setParticleIds(nParticles,sizeof(nParticles)/sizeof(Short_t));

    TString cuts = "RICHRINGCUTS,TOFPVSBETA,TOFINOPVSBETA,SHOWERSUMDIFFVSP";
    HPidAlgStandCuts *pid_alg_stand_cuts = new HPidAlgStandCuts(pid_ntuple.Data(),cuts.Data());
    pid_rec->addAlgorithm(pid_alg_stand_cuts);

    HPidParticleFiller *part_filler=new HPidParticleFiller("RUNGEKUTTA");
    part_filler->setAlgorithm(7); // 7-check values from HPidAlgStandCuts algorithm
    part_filler->print();


    //------------------------Pair creation--------------------------------
    HPairFiller *pair_filler=new HPairFiller("HPairFiller","HPairFiller");
    HPairFilter *pair_filter=new HPairFilter("HPairFilter","HPairFilter");
    pair_filter->setQAFileName((Char_t *)pair_filt_ntuple.Data());
 

   HTaskSet* tasks = gHades->getTaskSet( "simulation" );

   tasks->add( tof_task_set     ->make("simulation", "" ) );
   tasks->add( tofino_task_set  ->make("", "simulation" ) );
   tasks->add( rich_task_set    ->make("simulation", "noiseon") );
   tasks->add( rich_ipu_task_set->make("simulation" ) );
   tasks->add( shower_task_set  ->make("","simulation,leprecognition,lowshowerefficiency"));


   tasks->add( mdc_task_set     ->make( "rtdb" ) );

   tasks->add( tracking_tasks );
   tasks->add( new HPidTrackFiller( "makesimcategory,NOCHI2SEG0,NOCHI2SEG1" ) );  // whatever setup is wanted
   tasks->add( pidtofrec );
   tasks->add( new HPidTrackCleaner());                           // run standard track cleaner to get rough estimation of good tracks
   tasks->add( new HPidEvtInfoFiller() );                         // fill multiplicies per event
    tasks->add(pid_rec);
    tasks->add(part_filler);
    tasks->add(pair_filler);
    tasks->add(pair_filter);

   tasks->isTimed( kTRUE );

   // TASK LIST END
  
   simulation.initialize();
   //simulation.setupOutput();
   simulation.runEventLoop();
   simulation.finalize();
			   
}
