using namespace std;
#ifndef __CINT__

#include "TROOT.h"
#include "TString.h"
#include "TSystem.h"

#include "hdstutilities.h"
#include "hdstsimulation.h"

#include "hades.h"
#include "htaskset.h"
#include "hruntimedb.h"
#include "hdatasource.h"

#include "hstarttaskset.h"
#include "htriggertaskset.h"

#include "hrichIPUtaskset.h"
#include "hrichtaskset.h"

#include "htofinotaskset.h"
#include "htoftaskset.h"

#include "hshowertaskset.h"

#include "hwalltaskset.h"

#include "hmdcbitflipcor.h"
#include "hmdcsetup.h"
#include "hmdctaskset.h"
#include "hmdclookuptb.h"
#include "hmdc34clfinder.h"
#include "hmdctrackdset.h"
#include "hmdcdigitpar.h"
#include "hmdcdigitizer.h"
#include "hmdcvertexwriter.h"
#include "hmdcvertexfind.h"

#include "hrktrackBF.h"
#include "hsplinetaskset.h"
#include "hkicktaskset.h"
#include "hkicktrackbaseF.h"

#include "hpidtofrec.h"
#include "hpidalgstandcuts.h"
#include "hpidparticlefiller.h"
#include "hpidpdfmaker.h"
#include "hpidreconstructor.h"
#include "hpidskiptrigger.h"
#include "hpidtrackfiller.h"
#include "hpidevtinfofiller.h"
#include "hpidtrackcleaner.h"

#include "hpairfiller.h"
#include "hpairfilter.h"
//#include "hpairheaderfiller.h"

#include "hqamaker.h"

int main(int argc, char **argv)
{
   TROOT dst_production( "DstProductionApp", "Produce a DST File" );
   HDstSimulation simulation;
   
   simulation.configure( argc, argv );
#else


void DstSimulation(TString command_line)
{
   int                 argc = 0;
   char**              argv = NULL;
   HDstSimulation simulation;

   HDstUtilities::str2CmdLineArgs( command_line.Data(), &argc, &argv );
   simulation.configure( argc, argv );
   HDstUtilities::freeCmdLineArgs( argc, argv );
   
   
#endif

   simulation.printConfiguration();
   simulation.setupSpectrometer();
   simulation.setupInput();
   simulation.setupParameterSources();
   //simulation.setupParameterOutput();


   // TASK LIST

   // read-only access to the configuration
   const HDstProduction::DstConfiguration_t& dstCfg = simulation.getConfiguration();

   HRuntimeDb* rtdb=gHades->getRuntimeDb();

   HTriggerTaskSet      *triggerTaskSet      = new HTriggerTaskSet();
   
   HRichTaskSet         *richTaskSet         = new HRichTaskSet();
   HRichIPUTaskSet      *richIPUTaskSet      = new HRichIPUTaskSet();
   HShowerTaskSet       *showerTaskSet       = new HShowerTaskSet();
   HTofTaskSet          *tofTaskSet          = new HTofTaskSet();
   HTofinoTaskSet       *tofinoTaskSet       = new HTofinoTaskSet();
   HMdcTaskSet          *mdcTaskSet          = new HMdcTaskSet(); 
   
   //- New version of cluster finder -//
   HMdcLookUpTb::setUseFloatLevel();
   HMdc34ClFinder::setUseFloatLevel();

   HSplineTaskSet *splineTaskSet = new HSplineTaskSet("","");
   
 
   HTask *triggerTasks  = triggerTaskSet->make("simulation");
   HTask *richTasks     = richTaskSet   ->make("simulation","noiseon");
   HTask *richIPUTasks  = richIPUTaskSet->make("simulation");
   HTask *showerTasks   = showerTaskSet ->make("","simulation");
   HTask *tofTasks      = tofTaskSet    ->make("simulation");
   HTask *tofinoTasks   = tofinoTaskSet ->make( "", "simulation" );

   HTask *mdcTasks      = mdcTaskSet    ->make("rtdb","");
   HTask *splineTasks   = splineTaskSet ->make("","spline,reslowb,&old,tofclust,simulation,runge"); // all tracking tasks

   //------------------------PID TRACKCAND FILLER -----------------------------
   
   HTask  *pTrackFiller = new HPidTrackFiller("makesimcategory,nochi2seg1,nochi2seg0"); // no selection on chi2
  
   
   HPidTofRec *pidtofrec = new HPidTofRec("pidtofreconstructor","pidtofreconstructor","startrk,nopidparam","");
   pidtofrec->setQualityRichParams(150.,5.,5.,5.);
   
   HTaskSet * pidtofrectask = new HTaskSet("","");
   pidtofrectask->connect(pidtofrec);
   
    //------------------------NTuple names---------------------
    TString   pid_ntuple;
    TString   pair_filt_ntuple;

    // derive output and NTuple file name from input file name
    pid_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pid_ntuple.ReplaceAll( "_dst.root", 9, "_lep_nt.root", 12 );
    pid_ntuple.Prepend( "/" );
    pid_ntuple.Prepend( dstCfg.outputDir );

    pair_filt_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pair_filt_ntuple.ReplaceAll( "_dst.root", 9, "_pair_filt_nt.root", 18 );
    pair_filt_ntuple.Prepend( "/" );
    pair_filt_ntuple.Prepend( dstCfg.outputDir );

   
   //------------------------PARTICLE PID -----------------------------
   
   TString opt_pidrec="pdf,ALG_RUNGEKUTTA";
   HPidReconstructor *pPidRec=new HPidReconstructor(opt_pidrec.Data());

   Short_t nParticles[2]={2,3}; // only lepton ID
   pPidRec->setParticleIds(nParticles,sizeof(nParticles)/sizeof(Short_t));
   
   TString cuts="RICHRINGCUTS,TOFPVSBETA,TOFINOPVSBETA,SHOWERSUMDIFFVSP";
   HPidAlgStandCuts *pPidAlgStandCuts=new HPidAlgStandCuts(pid_ntuple.Data(),cuts.Data());
   
   pPidRec->addAlgorithm(pPidAlgStandCuts);
   
   HPidParticleFiller *pPartFiller=new HPidParticleFiller("RUNGEKUTTA");
   pPartFiller->setAlgorithm(7); // 7-check values from HPidAlgStandCuts algorithm
   pPartFiller->print();
   
   //------------------------ PAIRS -----------------------------
   
   HPairFiller *pPairFiller=new HPairFiller("HPairFiller","HPairFiller");
   HPairFilter *pPairFilter=new HPairFilter("HPairFilter","HPairFilter");
   pPairFilter->setQAFileName((Text_t*)pair_filt_ntuple.Data());
   
   //HPairHeaderFiller *pPairHeaderFiller=new HPairHeaderFiller("HPairHeaderFiller","HPairHeaderFiller");

   //------------------------ Master task set ---------------------------
   
   HTaskSet *masterTaskSetReal = gHades->getTaskSet("simulation");
   
   //masterTaskSetReal->add(triggerTasks);
   masterTaskSetReal->add(tofTasks);
   masterTaskSetReal->add(tofinoTasks);
   masterTaskSetReal->add(richTasks);
   masterTaskSetReal->add(richIPUTasks);
   masterTaskSetReal->add(showerTasks);
   masterTaskSetReal->add(mdcTasks);
   masterTaskSetReal->add(splineTasks);
   masterTaskSetReal->add(pTrackFiller);
   masterTaskSetReal->add(new HPidTrackCleaner("PidTrackCleaner","PidTrackCleaner"));
   masterTaskSetReal->add(new HPidEvtInfoFiller("PidEvtInfoFiller","PidEvtInfoFiller"));
   masterTaskSetReal->add(pidtofrectask);
   
   masterTaskSetReal->add(pPidRec);
   masterTaskSetReal->add(pPartFiller);
   masterTaskSetReal->add(pPairFiller);
   masterTaskSetReal->add(pPairFilter);
   //masterTaskSetReal->add(pPairHeaderFiller);

   masterTaskSetReal->isTimed(kTRUE);
   
   // TASK LIST END
   
   simulation.initialize();
   //simulation.setupOutput();
   simulation.runEventLoop();
   simulation.finalize();
			   
}
