using namespace std;
#ifndef __CINT__

#include "TROOT.h"
#include "TString.h"
#include "TSystem.h"

#include "hdstutilities.h"
#include "hdstsimulation.h"

#include "hades.h"
#include "htaskset.h"
#include "hruntimedb.h"
#include "hdatasource.h"

#include "hstarttaskset.h"
#include "htriggertaskset.h"

#include "hrichIPUtaskset.h"
#include "hrichtaskset.h"

#include "htofinotaskset.h"
#include "htoftaskset.h"

#include "hshowertaskset.h"

#include "hmdcbitflipcor.h"
#include "hmdcsetup.h"
#include "hmdctaskset.h"
#include "hmdclookuptb.h"
#include "hmdc34clfinder.h"
#include "hmdctrackdset.h"
#include "hmdcdigitpar.h"
#include "hmdcdigitizer.h"
#include "hmdcvertexwriter.h"
#include "hmdcvertexfind.h"

#include "hrktrackBF.h"
#include "hsplinetaskset.h"
#include "hkicktaskset.h"
#include "hkicktrackbaseF.h"

#include "hpidtofrec.h"
#include "hpidalgstandcuts.h"
#include "hpidparticlefiller.h"
#include "hpidpdfmaker.h"
#include "hpidreconstructor.h"
#include "hpidskiptrigger.h"
#include "hpidtrackfiller.h"
#include "hpidevtinfofiller.h"
#include "hpidtrackcleaner.h"

#include "hpairfiller.h"
#include "hpairfilter.h"

#include "hqamaker.h"

int main(int argc, char **argv)
{
   TROOT dst_production( "DstProductionApp", "Produce a DST File" );
   HDstSimulation simulation;
   
   simulation.configure( argc, argv );
#else


void DstSimulation(TString command_line)
{
   int                 argc = 0;
   char**              argv = NULL;
   HDstSimulation simulation;

   HDstUtilities::str2CmdLineArgs( command_line.Data(), &argc, &argv );
   simulation.configure( argc, argv );
   HDstUtilities::freeCmdLineArgs( argc, argv );
   
   
#endif

   simulation.printConfiguration();
   simulation.setupSpectrometer();
   simulation.setupInput();
   simulation.setupParameterSources();
   //simulation.setupParameterOutput();


   // TASK LIST (from Martin...)

   // read-only access to the configuration
   const HDstProduction::DstConfiguration_t& dstCfg = simulation.getConfiguration();

   HRuntimeDb* rtdb=gHades->getRuntimeDb();

   Short_t             nParticles[]        = {2, 3};
   HRichTaskSet*       rich_task_set          = new HRichTaskSet();
   HMdcTaskSet*        mdc_task_set        = new HMdcTaskSet();
   HTofTaskSet*        tof_task_set        = new HTofTaskSet();
   HTofinoTaskSet*     tofino_task_set     = new HTofinoTaskSet();
   HShowerTaskSet*     shower_task_set     = new HShowerTaskSet();
   HSplineTaskSet*     spline_task_set     = new HSplineTaskSet( "", "" );
   HTaskSet*           tasks               = gHades->getTaskSet( "simulation" );
   HTaskSet*           pair_analysis_tasks = NULL;
   HPidReconstructor*  pid_rec             = NULL;
   HPidParticleFiller* part_filler         = NULL;
   HPairFiller*        pair_filler         = NULL;
   HPairFilter*        pair_filter         = NULL;

   //////////////////////////////////////////////////////////////////
   //
   //       DST PART
   //
   //////////////////////////////////////////////////////////////////

   // use new MDC cluster finder with level adaption
   HMdcLookUpTb::setUseFloatLevel();        // inner MDCs
   HMdc34ClFinder::setUseFloatLevel();      // outer MDCs

   // this includes the following tasks:
   // Kick Tracking, Kick Track Clustering,
   // Spline Tracking, Spline Clustering,
   // Runge-Kutta Tracking, MetaMatch Procedure
   HTask* tracking_tasks = spline_task_set->make(
      "", "spline,reslowb,&old,tofclust,runge,simulation" );
   

   //------------------------NTuple names---------------------
    TString   pid_ntuple;
    TString   pair_filt_ntuple;

    // derive output and NTuple file name from input file name
    pid_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pid_ntuple.ReplaceAll( "_dst.root", 9, "_lep_nt.root", 12 );
    pid_ntuple.Prepend( "/" );
    pid_ntuple.Prepend( dstCfg.outputDir );

    pair_filt_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pair_filt_ntuple.ReplaceAll( "_dst.root", 9, "_pair_filt_nt.root", 18 );
    pair_filt_ntuple.Prepend( "/" );
    pair_filt_ntuple.Prepend( dstCfg.outputDir );
   

   //////////////////////////////////////////////////////////////////
   //
   //       PID RECONSTRUCTOR
   //
   //////////////////////////////////////////////////////////////////

   pid_rec = new HPidReconstructor("pdf, CL, merge, ALG_RUNGEKUTTA");
   TString cuts="RICHRINGCUTS,TOFPVSBETA,TOFINOPVSBETA,SHOWERSUMDIFFVSP";
   HPidAlgStandCuts *pPidAlgStandCuts=new HPidAlgStandCuts(pid_ntuple.Data(),cuts.Data());
   pid_rec->addAlgorithm( pPidAlgStandCuts );
   pid_rec->setParticleIds( nParticles, sizeof(nParticles) / sizeof(Short_t) );


   //////////////////////////////////////////////////////////////////
   //
   //       PARTICLE FILLER
   //
   //////////////////////////////////////////////////////////////////

   part_filler = new HPidParticleFiller( "RUNGEKUTTA,MAKESIMCATEGORY" );
   part_filler->setAlgorithm(7);
   part_filler->print();
  
   //////////////////////////////////////////////////////////////////
   //
   //       PAIRS
   //
   //////////////////////////////////////////////////////////////////

   pair_analysis_tasks = new HTaskSet( "Pair Analysis", "Pair Analysis" );
   pair_filler         = new HPairFiller( "HPairFiller", "HPairFiller" );
   pair_filter         = new HPairFilter( "HPairFilter", "HPairFilter" );
   pair_filter->setQAFileName(pair_filt_ntuple.Data());
 
   pair_analysis_tasks->add( pair_filler );
   pair_analysis_tasks->add( pair_filter );



   //////////////////////////////////////////////////////////////////
   //
   //       MASTER TASK LIST
   //
   //////////////////////////////////////////////////////////////////

   tasks->add( tof_task_set->make( "simulation" ) );
   tasks->add( tofino_task_set->make( "", "simulation" ) );
   tasks->add( rich_task_set->make( "simulation", "noiseon" ));
   tasks->add( shower_task_set->make(
      "", "simulation,leprecognition,lowshowerefficiency" ) );
   tasks->add( mdc_task_set->make( "rtdb" ) );
   tasks->add( tracking_tasks );
   tasks->add( new HPidTrackFiller( "makesimcategory,NOCHI2SEG1" ) );

   tasks->add(new HPidTrackCleaner("PidTrackCleaner","PidTrackCleaner"));
   tasks->add(new HPidEvtInfoFiller("PidEvtInfoFiller","PidEvtInfoFiller"));
   tasks->add(new HPidTofRec("pidtofreconstructor","pidtofreconstructor","sim",""));

   tasks->add( pid_rec     );
   tasks->add( part_filler );
   tasks->add( pair_analysis_tasks );

   tasks->isTimed( kTRUE );
   tasks->print();

   // TASK LIST END
  
   simulation.initialize();
   //simulation.setupOutput();
   simulation.runEventLoop();
   simulation.finalize();
			   
}
