using namespace std;
#ifndef __CINT__

#include "TROOT.h"
#include "TString.h"
#include "TSystem.h"

#include "hdstutilities.h"
#include "hdstsimulation.h"

#include "hades.h"
#include "htaskset.h"
#include "hruntimedb.h"
#include "hdatasource.h"

#include "hstarttaskset.h"
#include "htriggertaskset.h"

#include "hrichIPUtaskset.h"
#include "hrichtaskset.h"

#include "htofinotaskset.h"
#include "htoftaskset.h"

#include "hshowertaskset.h"

#include "hmdcbitflipcor.h"
#include "hmdcsetup.h"
#include "hmdctaskset.h"
#include "hmdclayercorrpar.h"
#include "hmdclookuptb.h"
#include "hmdc34clfinder.h"
#include "hmdctrackdset.h"
#include "hmdcdigitpar.h"
#include "hmdcdigitizer.h"
#include "hmdcvertexwriter.h"
#include "hmdcvertexfind.h"

#include "hrktrackBF.h"
#include "hsplinetaskset.h"
#include "hkicktaskset.h"
#include "hkicktrackbaseF.h"

#include "hpidtofrec.h"
#include "hpidalgstandcuts.h"
#include "hpidcorrelator.h"
#include "hpidevtfilter.h"
#include "hpidparticlefiller.h"
#include "hpidpdfmaker.h"
#include "hpidreconstructor.h"
#include "hpidskiptrigger.h"
#include "hpidtrackcandfilterrec.h"
#include "hpidtrackfiller.h"
#include "hpidevtinfofiller.h"
#include "hpidtrackcleaner.h"

#include "hpairfiller.h"
#include "hpairfilter.h"

#include "hqamaker.h"


Bool_t myselect(HPidTrackCand* pcand = NULL)
{
    // build in selection function for lepton candidates.
    // Requires besides an RICH hit, RK and fitted innerMDC
    // segment. 

    HPidTrackData* pTrack  = pcand  -> getTrackData();
    HPidHitData*    pHit   = pcand  -> getHitData();
    Bool_t test = kFALSE;
    if( pcand->isFlagAND(5,
			 HPidTrackCand::kIsAcceptedHitRICH,
			 HPidTrackCand::kIsAcceptedHitInnerMDC,
			 HPidTrackCand::kIsAcceptedHitOuterMDC,
			 HPidTrackCand::kIsAcceptedHitMETA,
			 HPidTrackCand::kIsAcceptedRK
			)
	&&
	pTrack->getRKChiSquare()     < 1000000
	) return kTRUE;

    return kFALSE;
  
// // 	&&
// // 	pHit->getInnerMdcChiSquare() > 0
// 	&&
// 	pTrack->getRKChiSquare()     < 1000000
// 	) test = kTRUE;

//     if(test)
//       {
// 	if(
// 	   fabs(pHit->getDeltaThetaRKRICH(pTrack->getRKTheta())) < 7.
// 	   &&
// 	   fabs(pHit->getDeltaPhiRKRICH(pTrack->getRKPhi(),pTrack->getRKTheta())) < 7.
// 	   )
// 	  {
// 	    return kTRUE;
// 	  }
// 	else return kFALSE;
//       }
//     else return kFALSE;
}

Bool_t myselecthadron(HPidTrackCand* pcand)
{
    // 16.05.08 selection for hadrons
    // build in selection function for hadron candidates.
    // Requires besides RK + META and fitted innerMDC.

    HPidTrackData* pTrack  = pcand  -> getTrackData();

    if( pcand->isFlagAND(4,
			 HPidTrackCand::kIsAcceptedHitInnerMDC,
			 HPidTrackCand::kIsAcceptedHitOuterMDC,
			 HPidTrackCand::kIsAcceptedHitMETA,
			 HPidTrackCand::kIsAcceptedRK
			)
       //&&
       //pHit->getInnerMdcChiSquare() > 0
       &&
       pTrack->getRKChiSquare()     < 1000000
      ) return kTRUE;

    return kFALSE;
}




int main(int argc, char **argv)
{
   TROOT dst_production( "DstProductionApp", "Produce a DST File" );
   HDstSimulation simulation;
   
   simulation.configure( argc, argv );
#else


void DstSimulation(TString command_line)
{
   int                 argc = 0;
   char**              argv = NULL;
   HDstSimulation simulation;

   HDstUtilities::str2CmdLineArgs( command_line.Data(), &argc, &argv );
   simulation.configure( argc, argv );
   HDstUtilities::freeCmdLineArgs( argc, argv );
   
   
#endif

   simulation.printConfiguration();
   simulation.setupSpectrometer();
   simulation.setupInput();
   simulation.setupParameterSources();
   //simulation.setupParameterOutput();


   // TASK LIST (like for real pair ntuples)

   // read-only access to the configuration
   const HDstProduction::DstConfiguration_t& dstCfg = simulation.getConfiguration();

   HRuntimeDb* rtdb=gHades->getRuntimeDb();

   HTofTaskSet*     tof_task_set        = new HTofTaskSet();
   HTofinoTaskSet*  tofino_task_set     = new HTofinoTaskSet();
   HRichTaskSet*    rich_task_set       = new HRichTaskSet();
   HMdcTaskSet*     mdc_task_set        = new HMdcTaskSet();
   HShowerTaskSet*  shower_task_set     = new HShowerTaskSet();
   HSplineTaskSet*  spline_task_set     = new HSplineTaskSet( "", "" );

   //- New version of cluster finder -//
   HMdcLookUpTb::setUseFloatLevel();
   HMdc34ClFinder::setUseFloatLevel();

   HMdcLayerCorrPar* fMdcLayerCorrPar = (HMdcLayerCorrPar*)gHades->getRuntimeDb()->getContainer("MdcLayerCorrPar");
   fMdcLayerCorrPar -> setDefaultPar();
   fMdcLayerCorrPar -> setStatic();
   //--------------------------------------------------------------------

  HTask *richTasks     = rich_task_set   ->make("simulation","noiseon");
  HTask *showerTasks   = shower_task_set ->make("", "simulation,leprecognition,lowshowerefficiency");
  HTask *tofTasks      = tof_task_set    ->make("simulation");
  HTask *tofinoTasks   = tofino_task_set ->make( "", "simulation");
  HTask *mdcTasks      = mdc_task_set    ->make("rtdb","");
  HTask *splineTasks   = spline_task_set ->make("","spline,reslowb,&old,tofclust,simulation,runge");

  //------------------------PID TRACKCAND FILLER -----------------------------

  HTask  *pTrackFiller = new HPidTrackFiller("makesimcategory,nochi2seg1,nochi2seg0"); // no selection on chi2


  //------------------------ TRACK SELCTION ---------------------------------

  HPidTrackCleaner* cleaner = new HPidTrackCleaner();
  HPidTrackSorter::setIgnoreRICH();
  //HPidTrackSorter::setDebug();
  //HPidTrackSorter::setPrintLevel(3);
  //HPidTrackSorter::setIgnoreInnerMDC();
  cleaner->setUserSelectionLeptons(myselect);
  cleaner->setUserSelectionHadrons(myselecthadron);
  
  HPidEvtFilter *myEvtFilter = new HPidEvtFilter("MyEvtFilter","MyEvtFilter",kFALSE,"select",0);

  HPidTofRec *pidtofrec = new HPidTofRec("pidtofreconstructor","pidtofreconstructor","sim,startrk","");
  HTaskSet * pidtofrectask = new HTaskSet("","");
  pidtofrectask->connect(pidtofrec);


    //------------------------NTuple names---------------------
    TString   pid_ntuple;
    TString   pair_filt_ntuple;

    // derive output and NTuple file name from input file name
    pid_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pid_ntuple.ReplaceAll( "_dst.root", 9, "_lep_nt.root", 12 );
    pid_ntuple.Prepend( "/" );
    pid_ntuple.Prepend( dstCfg.outputDir );

    pair_filt_ntuple = gSystem->BaseName( const_cast<char*>(dstCfg.outputFile.Data()) );
    pair_filt_ntuple.ReplaceAll( "_dst.root", 9, "_pair_filt_nt.root", 18 );
    pair_filt_ntuple.Prepend( "/" );
    pair_filt_ntuple.Prepend( dstCfg.outputDir );
   
   
   //------------------------PARTICLE PID -----------------------------

    TString opt_pidrec="pdf,CL,merge,ALG_RUNGEKUTTA";
   HPidReconstructor *pPidRec=new HPidReconstructor(opt_pidrec.Data());
   
   Short_t nParticles[2]={2,3}; // only lepton ID
   pPidRec->setParticleIds(nParticles,sizeof(nParticles)/sizeof(Short_t));
   
   TString cuts="RICHRINGCUTS,TOFPVSBETA,TOFINOPVSBETA,SHOWERSUMDIFFVSP";
   HPidAlgStandCuts *pPidAlgStandCuts=new HPidAlgStandCuts(pid_ntuple.Data(),cuts.Data());
   
   pPidRec->addAlgorithm(pPidAlgStandCuts);
   
   HPidParticleFiller *pPartFiller=new HPidParticleFiller("RUNGEKUTTA");
   pPartFiller->setAlgorithm(7); // 7-check values from HPidAlgStandCuts algorithm
   
   pPartFiller->print();

   //------------------------ PAIRS -----------------------------
   
   HPairFiller *pPairFiller=new HPairFiller("HPairFiller","HPairFiller");
   HPairFilter *pPairFilter=new HPairFilter("HPairFilter","HPairFilter");
   pPairFilter->setQAFileName(pair_filt_ntuple.Data());
   
  


   //------------------------ Master task set ---------------------------
   HTaskSet *masterTaskSetSim = gHades->getTaskSet("simulation");
   
   masterTaskSetSim->add(richTasks);
   masterTaskSetSim->add(tofTasks);
   masterTaskSetSim->add(tofinoTasks);
   masterTaskSetSim->add(showerTasks);
   masterTaskSetSim->add(mdcTasks);
   masterTaskSetSim->add(splineTasks);
   masterTaskSetSim->add(pTrackFiller);
   masterTaskSetSim->add(new HPidTrackCleaner("PidTrackCleaner","PidTrackCleaner"));
   masterTaskSetSim->add(new HPidEvtInfoFiller("PidEvtInfoFiller","PidEvtInfoFiller"));
   masterTaskSetSim->add(cleaner);
   masterTaskSetSim->add(myEvtFilter);
   masterTaskSetSim->add(pidtofrectask);  
   masterTaskSetSim->add(new HPidTrackCandFilterRec("RICHHIT"));
   masterTaskSetSim->add(new HPidCorrelator(1.0));
   masterTaskSetSim->add(pPidRec);
   masterTaskSetSim->add(pPartFiller);
   masterTaskSetSim->add(pPairFiller);
   masterTaskSetSim->add(pPairFilter);
   
   
   masterTaskSetSim->isTimed( kTRUE );
   
   
   // TASK LIST END
   
   simulation.initialize();
   //simulation.setupOutput();
   simulation.runEventLoop();
   simulation.finalize();
			   
}

